@@ -1,6 +1,6 @@
 package Module::Load;
 
-$VERSION = "-1";
+$VERSION = '0.18';
 
 use strict;
 use File::Spec ();
@@ -30,18 +30,17 @@ sub load (*;@)  {
             die $err if $err;
         }
     }
-    __PACKAGE__->_export_to_level(1, $mod, @_) if @_;
-}
 
-### 5.004's Exporter doesn't have export_to_level.
-### Taken from Michael Schwerns Test::More and slightly modified
-sub _export_to_level {
-    my $pkg     = shift;
-    my $level   = shift;
-    my $mod     = shift;
-    my $callpkg = caller($level);
-
-    $mod->export($callpkg, @_);
+    ### This addresses #41883: Module::Load cannot import
+    ### non-Exporter module. ->import() routines weren't
+    ### properly called when load() was used.
+    {   no strict 'refs';
+        my $import;
+        if (@_ and $import = $mod->can('import')) {
+            unshift @_, $mod;
+            goto &$import;
+        }
+    }
 }
 
 sub _to_file{
@@ -57,6 +56,13 @@ sub _to_file{
 
     $file   .= '.pm' if $pm;
 
+    ### on perl's before 5.10 (5.9.5@31746) if you require
+    ### a file in VMS format, it's stored in %INC in VMS
+    ### format. Therefor, better unixify it first
+    ### Patch in reply to John Malmbergs patch (as mentioned
+    ### above) on p5p Tue 21 Aug 2007 04:55:07
+    $file = VMS::Filespec::unixify($file) if $^O eq 'VMS';
+
     return $file;
 }
 
@@ -89,15 +95,15 @@ Module::Load - runtime require of both modules and files
     load Data::Dumper;      # loads that module
     load 'Data::Dumper';    # ditto
     load $module            # tritto
-    
+
     my $script = 'some/script.pl'
     load $script;
     load 'some/script.pl';	# use quotes because of punctuations
-    
+
     load thing;             # try 'thing' first, then 'thing.pm'
 
     load CGI, ':standard'   # like 'use CGI qw[:standard]'
-    
+
 
 =head1 DESCRIPTION
 
@@ -135,9 +141,9 @@ If the argument matches only C<[\w:']>, it must be a module
 =item *
 
 If the argument matches only C<\w>, it could either be a module or a
-file. We will try to find C<file> first in C<@INC> and if that fails,
-we will try to find C<file.pm> in @INC.
-If both fail, we die with the respective error messages.
+file. We will try to find C<file.pm> first in C<@INC> and if that
+fails, we will try to find C<file> in @INC.  If both fail, we die with
+the respective error messages.
 
 =back
 
@@ -154,20 +160,22 @@ C<Module::Load> cannot do implicit imports, only explicit imports.
 to import from a module, even if the functions are in that modules'
 C<@EXPORT>)
 
+=head1 ACKNOWLEDGEMENTS
+
+Thanks to Jonas B. Nielsen for making explicit imports work.
+
+=head1 BUG REPORTS
+
+Please report bugs or other issues to E<lt>bug-module-load@rt.cpan.org<gt>.
+
 =head1 AUTHOR
 
 This module by Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
-Thanks to Jonas B. Nielsen for making explicit imports work.
-
 =head1 COPYRIGHT
 
-This module is
-copyright (c) 2002 Jos Boumans E<lt>kane@cpan.orgE<gt>.
-All rights reserved.
+This library is free software; you may redistribute and/or modify it
+under the same terms as Perl itself.
 
-This library is free software;
-you may redistribute and/or modify it under the same
-terms as Perl itself.
 
-=cut                               
+=cut
